from pydantic import BaseModel, Field
from typing import Literal


class GetArynDocumentExtractedPropertiesModel(BaseModel):
    """
    Input schema for get_aryn_document()

    Attributes:
        docset_id
        doc_id
        output_format
    """

    docset_id: str = Field(
        ...,
        description="""
            docset_id (str, required)
            The unique identifier of the DocSet containing the document""",
    )

    doc_id: str = Field(
        ...,
        description="""
            doc_id (str, required)
            The unique identifier of the document to retrieve""",
    )

    output_format: Literal["json", "csv"] = Field(
        ...,
        description="""
            output_format (str, required)
            The format to save the extracted properties in. There are two possible values:
            json
            csv""",
    )



from pydantic import BaseModel, Field


class QueryArynDocSetModel(BaseModel):
    """
    Input schema for query_aryn_docset()

    Attributes:
        docset_id (str, required)
        query (str, required)
        summarize_result (bool, optional)
    """

    docset_id: str = Field(
        ...,
        description="""
            docset_id (str, required)
            The id of the docset to query.""",
    )

    query: str = Field(
        ...,
        description="""
            query (str, required)
            The query to search for in the docset.  """,
    )

    summarize_result: bool = Field(
        False,
        description="""
            summarize_result (bool, optional)
            Whether to summarize the result of the query.""",
    )



import os
import sys
import json
import time
import zipfile
import pandas as pd

from PIL import Image
from pathlib import Path
from functools import wraps
from typing import Dict, Union, List
from io import BytesIO, StringIO


def get_output_dir() -> Path:
    output_dir = os.environ.get("ARYN_MCP_OUTPUT_DIR")
    if output_dir:
        path = Path(output_dir).resolve()
    else:
        project_root = Path(__file__).parent.parent.parent.resolve()
        path = project_root / "temp"

    path.mkdir(parents=True, exist_ok=True)
    return path.resolve()  # Return absolute path


def save_file(
    data: Union[Dict, str, Image.Image, bytes],
    filename: str,
    output_format: str = "json",
) -> Path:
    try:
        base_dir = get_output_dir()  # This is now an absolute path
        path = ensure_unique_filename(base_dir / filename, f".{output_format}")
        if isinstance(data, dict) or isinstance(data, list) and output_format == "json":
            with open(path, "w", encoding="utf-8") as f:
                json.dump(data, f, indent=4)
        elif isinstance(data, str) and output_format == "markdown":
            with open(path, "w", encoding="utf-8") as f:
                f.write(data)
        elif isinstance(data, dict) and output_format == "markdown":
            with open(path, "w", encoding="utf-8") as f:
                f.write(data.get("markdown", ""))
        elif isinstance(data, Image.Image) and output_format in ["png", "jpg", "jpeg"]:
            data.save(path, format=output_format.upper())
        elif isinstance(data, bytes) and output_format == "zip":
            with open(path, "wb") as f:
                f.write(data)
        else:
            raise ValueError(f"Unsupported combination of data type {type(data)} and format {output_format}")

        return path.resolve()  # Return absolute path

    except Exception as e:
        raise e


def ensure_unique_filename(base_path: Union[str, Path], extension: str) -> Path:
    base_path = Path(base_path)
    counter = 1
    new_path = base_path.with_suffix(extension)

    while new_path.exists():
        new_path = base_path.parent / f"{base_path.stem}_{counter}{extension}"
        counter += 1

    return new_path


def create_zip_from_dataframes(dataframes: List[pd.DataFrame]) -> bytes:
    zip_buffer = BytesIO()

    with zipfile.ZipFile(zip_buffer, "w", zipfile.ZIP_DEFLATED) as zip_file:
        for i, df in enumerate(dataframes):
            if df is not None and not df.empty:
                csv_buffer = StringIO()
                df.to_csv(csv_buffer, index=False)
                csv_content = csv_buffer.getvalue()

                csv_filename = f"table_{i+1}.csv"
                zip_file.writestr(csv_filename, csv_content)

    zip_buffer.seek(0)
    return zip_buffer.getvalue()


def timing_decorator(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        end_time = time.time()

        print(f"{func.__name__} took {end_time - start_time:.2f} seconds", file=sys.stderr)
        return result

    return wrapper



from pydantic import BaseModel, Field


class DeleteArynDocSetPropertiesModel(BaseModel):
    """
    Input schema for delete_aryn_docset_properties()

    Attributes:
        docset_id
        schema
    """

    docset_id: str = Field(
        ...,
        description="""
            docset_id (str, required)
            The unique identifier of the DocSet containing the document""",
    )

    properties_to_delete: list[str] = Field(
        ...,
        description="""
            properties_to_delete (list[str], required)
            A list of names of properties to delete""",
    )



from pydantic import BaseModel, Field
from os import PathLike


class AddArynDocumentModel(BaseModel):
    """
    Input schema for add_aryn_document()

    Attributes:
        file
        docset_id
    """

    file: str | PathLike = Field(
        ...,
        description="""
            file (str | PathLike, required)
            A file opened in binary mode or a path specified as either a str or PathLike instance indicating the document to add. The path
            can either be a local path or an Amazon S3 url starting with s3://. In the latter case, you must have boto3 installed and AWS
            credentials set up in your environment""",
    )

    docset_id: str = Field(
        ...,
        description="""
            docset_id (str, required)
            The id of the DocSet into which to add the document""",
    )



from pydantic import BaseModel, Field
from typing import Literal


class PropertiesFilterModel(BaseModel):
    """
    Schema for defining properties to be filtered

    Attributes:
        property
        value
        operator
    """

    property: str = Field(
        ...,
        description="""
            property (str, required)
            The property to be filtering for in a DocSet""",
    )

    value: str = Field(
        ...,
        description="""
            value (str, required)
            The value of the property to be filtering for in a DocSet""",
    )

    property_type: Literal["str", "int", "bool"] = Field(
        ...,
        description="""
            property_type (str, required)
            The type of the property specified in the DocSet. It MUST match the type of the property in the DocSet. There are 3 possible values:
            str:  A string
            int:  A number
            bool: A boolean
            """,
    )

    operator: Literal["=", ">", "<", ">=", "<=", "<>", "like"] = Field(
        ...,
        description="""
            operator (str, required)
            The comparison operator for the property being filtered. There are 7 possible values:
            like:  Documents or elements where the property contains the value
            =:     Documents or elements where the property is equal to the value
            >:     Documents or elements where the property is greater than the value
            <:     Documents or elements where the property is less than the value
            >=:    Documents or elements where the property is greater than or equal to the value
            <=:    Documents or elements where the property is less than or equal to the value
            <>:    Documents or elements where the property is not the value""",
    )



from pydantic import BaseModel, Field


class GetArynDocSetModel(BaseModel):
    """
    Input schema for get_aryn_docset()

    Attributes:
        docset_id
    """

    docset_id: str = Field(
        ...,
        description="""
            docset_id (str, required)
            The unique identifier of the DocSet to retrieve""",
    )



from aryn_sdk.client import Client
from aryn_sdk.client.client import SearchRequest

from aryn_sdk.types.query import Query
from aryn_sdk.types.schema import Schema, NamedProperty

from typing import Literal
from .models import PropertiesFilterModel

from collections import defaultdict


class ArynDocSetManager:
    def __init__(self, aryn_api_key: str | None = None, aryn_url: str = "https://api.aryn.ai"):
        if aryn_api_key and aryn_url:
            self.client = Client(aryn_api_key=aryn_api_key, aryn_url=aryn_url)
        else:
            self.client = Client()

    def _parse_through_schema(self, schema: list[NamedProperty]):
        properties_list = [vars(property) for property in schema]
        return properties_list

    def _generate_docset_info(self, docset, listing: bool = False, exclude_schema: bool = False) -> dict:
        docset_params = docset
        if not listing:
            docset_params = docset.value

        docset_info = {
            "docset_id": docset_params.docset_id,
            "name": docset_params.name,
            "readonly": docset_params.readonly,
            "properties": docset_params.properties,
            "schema": (self._parse_through_schema(docset_params.schema_.properties) if docset_params.schema_ else None),
            "size": docset_params.size,
        }

        if exclude_schema:
            docset_info.pop("schema")

        return docset_info

    def _generate_properties_filter_string(self, properties_filter: list[PropertiesFilterModel]):
        properties_filter_string = ""
        for filter in properties_filter:
            if filter.property_type == "str" or filter.property_type == "bool":
                properties_filter_string += (
                    f"""(properties.entity."{filter.property}"{filter.operator}"{filter.value}") AND """
                )
            elif filter.property_type == "int":
                properties_filter_string += (
                    f"""(properties.entity."{filter.property}"{filter.operator}{filter.value}) AND """
                )

        properties_filter_string = properties_filter_string[:-5]

        return properties_filter_string

    def create_docset(self, name: str, schema: Schema | None) -> dict:
        try:
            docset = self.client.create_docset(name=name, schema=schema)
            created_docset_info = self._generate_docset_info(docset)

            return created_docset_info

        except Exception as e:
            raise Exception(f"Failed to create docset {name}: {str(e)}") from e

    def get_docset(self, docset_id: str, exclude_schema: bool = False) -> dict | None:
        try:
            docset = self.client.get_docset(docset_id=docset_id)
            docset_info = self._generate_docset_info(docset, exclude_schema=exclude_schema)

            return docset_info

        except Exception as e:
            if "404" in str(e) or "not found" in str(e).lower():
                return None
            raise Exception(f"Failed to get docset {docset_id}: {str(e)}") from e

    def delete_docset(self, docset_id: str) -> dict:
        try:
            docset = self.client.delete_docset(docset_id=docset_id)
            deleted_docset_info = self._generate_docset_info(docset)

            return deleted_docset_info

        except Exception as e:
            if "404" in str(e) or "not found" in str(e).lower():
                raise Exception(f"Docset {docset_id} not found: {str(e)}") from e
            raise Exception(f"Failed to delete docset {docset_id}: {str(e)}") from e

    def list_docsets(self, page_size: int, name_eq: str | None = None, page_token: str | None = None) -> list[dict]:
        try:
            if name_eq:
                docsets = self.client.list_docsets(
                    page_size=page_size, name_eq=name_eq, page_token=page_token
                ).get_all()
            else:
                docsets = self.client.list_docsets(page_size=page_size).get_all()

            docsets_info = [self._generate_docset_info(d, True, True) for d in docsets]
            return docsets_info

        except Exception as e:
            raise Exception(f"Failed to list docsets: {str(e)}") from e

    def extract_properties(self, docset_id: str, properties_to_extract: Schema) -> dict:
        try:
            # NOTE: Extracting a property from an empty docset will do nothing, pending fix from Aryn
            docset_info = self.get_docset(docset_id)
            if not docset_info or docset_info["size"] is None or docset_info["size"] == 0:
                raise Exception(f"Docset {docset_id} is empty, cannot extract properties")

            result = self.client.extract_properties(docset_id=docset_id, schema=properties_to_extract)
            result = result.value

            if result.exit_status == 0:
                return {
                    "docset_id": docset_id,
                    "extracted_properties": [p.name for p in properties_to_extract.fields],
                }
            else:
                raise Exception(f"Exit status: {result.exit_status}")

        except Exception as e:
            raise Exception(f"Failed to extract properties for docset {docset_id}: {str(e)}") from e

    def delete_properties(self, docset_id: str, properties_to_delete: list[str]) -> dict:

        try:
            result = self.client.delete_properties(docset_id=docset_id, property_names=properties_to_delete)

            result = result.value

            if result.exit_status == 0:
                return {
                    "docset_id": docset_id,
                    "deleted_properties": properties_to_delete,
                }
            else:
                raise Exception(f"Exit status: {result.exit_status}")
        except Exception as e:
            raise Exception(f"Failed to delete properties for docset {docset_id}: {str(e)}") from e

    def search(
        self,
        docset_id: str,
        query_or_properties_filter: Literal["query", "properties_filter"],
        query: str | None,
        query_type: Literal["keyword", "vector", "lexical", "hybrid"] | None,
        properties_filter: list[PropertiesFilterModel] | None,
        page_size: int,
        return_type: Literal["doc", "element"],
        page_token: str | None,
    ) -> dict:

        properties_filter_string = None
        if properties_filter is not None:
            properties_filter_string = self._generate_properties_filter_string(properties_filter=properties_filter)

        try:
            if query_or_properties_filter == "query":
                search_result = self.client.search(
                    docset_id=docset_id,
                    query=SearchRequest(
                        query=query,
                        query_type=query_type,
                        include_fields=["doc_id"],
                        return_type=return_type,
                    ),
                    page_size=page_size,
                )
            elif query_or_properties_filter == "properties_filter":
                search_result = self.client.search(
                    docset_id=docset_id,
                    query=SearchRequest(
                        properties_filter=properties_filter_string,
                        include_fields=["doc_id"],
                        return_type=return_type,
                    ),
                    page_size=page_size,
                )

            search_result = search_result.value

            return {
                "results": search_result.results,
                "next_page_token": search_result.next_page_token,
            }
        except Exception as e:
            raise Exception(f"Failed to search docset {docset_id}: {str(e)}") from e

    def query(self, docset_id: str, query: str, summarize_result: bool):

        try:
            query_result = self.client.query(
                query=Query(
                    query=query,
                    docset_id=docset_id,
                    summarize_result=summarize_result,
                    stream=True,
                )
            )

            query_result_data = defaultdict(str)
            for event in query_result:
                if event.event_type == "trace_doc":
                    query_result_data["doc_id"] = event.data.doc["doc_id"]
                elif event.event_type == "result_summary":
                    query_result_data["summary"] += event.data
                elif event.event_type == "complete" and event.data[-9:] == "0 results":
                    query_result_data["summary"] = "No results found"
                    query_result_data["doc_id"] = "No results found"

            return dict(query_result_data)
        except Exception as e:
            raise Exception(f"Failed to query docset {docset_id}: {str(e)}") from e



from .create_aryn_docset_model import CreateArynDocSetModel
from .draw_boxes_model import DrawBoxesModel
from .partition_model import PartitionModel
from .get_aryn_docset_model import GetArynDocSetModel
from .list_aryn_docsets_model import ListArynDocSetsModel
from .delete_aryn_docset_model import DeleteArynDocSetModel
from .add_aryn_document_model import AddArynDocumentModel
from .list_aryn_documents_model import ListArynDocumentsModel
from .get_aryn_document_extracted_properties_model import (
    GetArynDocumentExtractedPropertiesModel,
)
from .delete_aryn_document_model import DeleteArynDocumentModel
from .extract_aryn_document_properties_model import ExtractArynDocumentPropertiesModel
from .delete_aryn_docset_properties_model import DeleteArynDocSetPropertiesModel
from .properties_filter_model import PropertiesFilterModel
from .search_aryn_docset_model import SearchArynDocSetModel
from .query_aryn_docset_model import QueryArynDocSetModel
from .document_schema import Schema
from .get_aryn_document_components_model import GetArynDocumentComponentsModel
from .draw_boxes_model import PageRange

__all__ = [
    "CreateArynDocSetModel",
    "DrawBoxesModel",
    "PartitionModel",
    "GetArynDocSetModel",
    "ListArynDocSetsModel",
    "DeleteArynDocSetModel",
    "AddArynDocumentModel",
    "ListArynDocumentsModel",
    "GetArynDocumentComponentsModel",
    "GetArynDocumentExtractedPropertiesModel",
    "DeleteArynDocumentModel",
    "ExtractArynDocumentPropertiesModel",
    "DeleteArynDocSetPropertiesModel",
    "PropertiesFilterModel",
    "SearchArynDocSetModel",
    "Schema",
    "QueryArynDocSetModel",
    "PageRange",
]



from pydantic import BaseModel, Field


class ListArynDocumentsModel(BaseModel):
    """
    Input schema for list_aryn_documents()

    Attributes:
        docset_id
        page_size
        page_token
    """

    docset_id: str = Field(
        ...,
        description="""
            docset_id (str, required)
            ID of the DocSet containing the documents""",
    )

    page_size: int = Field(
        100,
        description="""
            page_size (int, optional)
            Number of items per page""",
    )

    page_token: str | None = Field(
        None,
        description="""
            page_token (str, optional)
            Token for pagination. If provided, this will indicate to the server where to begin the next set
            of records to return. Valid for 24 hours.""",
    )



from pydantic import BaseModel, Field


class GetArynDocumentComponentsModel(BaseModel):
    """
    Input schema for get_aryn_document_elements(), get_aryn_document_tables(), and get_aryn_document_original_file()

    Attributes:
        return_original_elements
        docset_id
        doc_id
    """

    docset_id: str = Field(
        ...,
        description="""
            docset_id (str, required)
            The unique identifier of the DocSet containing the document""",
    )

    doc_id: str = Field(
        ...,
        description="""
            doc_id (str, required)
            The unique identifier of the document to retrieve""",
    )

    return_original_elements: bool = Field(
        ...,
        description="""
            return_original_elements (bool, required)
            Whether to return the original elements or the parsed elements""",
    )



from os import PathLike
from .models import PartitionModel

from aryn_sdk.client import Client


class ArynDocumentManager:
    def __init__(self, aryn_api_key: str | None = None, aryn_url: str = "https://api.aryn.ai"):
        if aryn_api_key and aryn_url:
            self.client = Client(aryn_api_key=aryn_api_key, aryn_url=aryn_url)
        else:
            self.client = Client()

    def _create_doc_info(self, doc, listing: bool = False) -> dict:
        doc_params = doc

        if not listing:
            doc_params = doc.value

        doc_dict = {
            "account_id": doc_params.account_id,
            "doc_id": doc_params.doc_id,
            "name": doc_params.name,
            "size": doc_params.size,
            "content_type": doc_params.content_type,
            "properties": doc_params.properties,
        }

        return doc_dict

    def _extract_properties(self, properties):
        if not isinstance(properties, dict):
            return properties

        if "entity" in properties:
            return self._extract_properties(properties["entity"])

        return properties

    def _create_partition_options(self, options: PartitionModel):
        partition_options = {
            "threshold": options.threshold,
            "text_mode": options.text_mode,
            "table_mode": options.table_mode,
            "text_extraction_options": {"remove_line_breaks": options.remove_line_breaks},
            "table_extraction_options": {"include_additional_text": options.include_additional_text},
            "extract_images": options.extract_images,
            "extract_image_format": options.extract_image_format,
            "summarize_images": options.summarize_images,
            "selected_pages": options.selected_pages,
            "chunking_options": {
                "strategy": options.strategy,
                "max_tokens": options.max_tokens,
                "tokenizer": options.tokenizer,
                "merge_across_pages": options.merge_across_pages,
            },
            "output_label_options": {
                "promote_title": options.promote_title,
                "title_candidate_elements": options.title_candidate_elements,
                "orientation_correction": options.orientation_correction,
            },
            "markdown_options": {
                "include_pagenum": options.include_pagenum,
                "include_headers": options.include_headers,
                "include_footers": options.include_footers,
            },
        }
        return partition_options

    def add_document(self, file: str | PathLike, docset_id: str, options: PartitionModel):
        try:
            partition_options = self._create_partition_options(options)
            doc = self.client.add_doc(file=file, docset_id=docset_id, options=partition_options)
            doc_info = self._create_doc_info(doc)
            return doc_info
        except Exception as e:
            raise Exception(f"Failed to add document to docset {docset_id}: {str(e)}") from e

    def list_documents(self, docset_id: str, page_size: int, page_token: str | None):
        try:
            docs = self.client.list_docs(docset_id=docset_id, page_size=page_size, page_token=page_token)
            docs_info = [self._create_doc_info(doc, listing=True) for doc in docs]
            for doc_info in docs_info:
                doc_id = doc_info["doc_id"]
                extracted_properties = self.get_document(
                    docset_id=docset_id, doc_id=doc_id, include_elements=False, include_binary=False
                )["properties"]
                doc_info["extracted_properties"] = extracted_properties
            return docs_info
        except Exception as e:
            raise Exception(f"Failed to list documents in docset {docset_id}: {str(e)}") from e

    def get_document(self, docset_id: str, doc_id: str, include_elements: bool, include_binary: bool):
        try:
            doc = self.client.get_doc(
                docset_id=docset_id,
                doc_id=doc_id,
                include_elements=include_elements,
                include_binary=include_binary,
            )
            doc = doc.value

            element_dict = {}
            if include_elements:
                for element in doc.elements:
                    element_dict[element.id] = {
                        "id": element.id,
                        "type": element.type,
                        "text_representation": element.text_representation,
                        "properties": element.properties,
                        "bbox": element.bbox,
                    }

            properties = self._extract_properties(doc.properties)

            doc_info = {
                "doc_id": doc.id,
                "elements": element_dict,
                "properties": properties,
                "binary_data": doc.binary_data,
                "original_elements": doc.properties["_original_elements"],
            }
            return doc_info
        except Exception as e:
            if "404" in str(e) or "not found" in str(e).lower():
                return None
            raise Exception(f"Failed to get document {doc_id} in docset {docset_id}: {str(e)}") from e

    def delete_document(self, docset_id: str, doc_id: str):
        try:
            doc = self.client.delete_doc(docset_id=docset_id, doc_id=doc_id)
            doc_info = self._create_doc_info(doc)
            return doc_info
        except Exception as e:
            if "404" in str(e) or "not found" in str(e).lower():
                raise Exception(f"Document {doc_id} not found in docset {docset_id}") from e
            raise Exception(f"Failed to delete document {doc_id} in docset {docset_id}: {str(e)}") from e

    def get_document_binary(self, docset_id: str, doc_id: str, file_path: str | PathLike):
        try:
            self.client.get_doc_binary(docset_id=docset_id, doc_id=doc_id, file=file_path)
            return file_path
        except Exception as e:
            raise Exception(f"Failed to get document binary for {doc_id} in docset {docset_id}: {str(e)}") from e



import datetime
from enum import Enum
import json
import logging
from typing import Annotated, Any, Literal, Optional, TypeAlias
from pydantic import (
    AliasChoices,
    BaseModel,
    ConfigDict,
    Field,
    TypeAdapter,
    ValidatorFunctionWrapHandler,
    WrapValidator,
    ValidationError,
)


logger = logging.getLogger(__name__)


class SchemaField(BaseModel):
    """Represents a field in a DocSet schema."""

    name: str
    """The name of the field."""

    field_type: str
    """The type of the field."""

    default: Optional[Any] = None
    """The default value for the field."""

    description: Optional[str] = None
    """A natural language description of the field."""

    examples: Optional[list[Any]] = None
    """A list of example values for the field."""


class Schema(BaseModel):
    """Represents the schema of a DocSet."""

    fields: list[SchemaField]
    """A list of fields belong to this schema."""


class DataType(str, Enum):
    BOOL = "bool"
    INT = "int"
    FLOAT = "float"
    STRING = "string"
    DATE = "date"
    DATETIME = "datetime"
    ARRAY = "array"
    CUSTOM = "custom"
    CHOICE = "choice"
    OBJECT = "object"

    @classmethod
    def values(cls):
        return set(map(lambda c: c.value, cls))

    @classmethod
    def from_python(cls, python_val: Any) -> "DataType":
        return cls.from_python_type(type(python_val))

    @classmethod
    def from_python_type(cls, python_type: type) -> "DataType":
        """Convert a Python type to a DataType."""
        if python_type is bool:
            return cls.BOOL
        elif python_type is int:
            return cls.INT
        elif python_type is float:
            return cls.FLOAT
        elif python_type is str:
            return cls.STRING
        elif python_type is datetime.date:
            return cls.DATE
        elif python_type is datetime.datetime:
            return cls.DATETIME
        elif python_type is list:
            return cls.ARRAY
        elif python_type is dict:
            return cls.OBJECT
        else:
            logger.warning(f"Unsupported Python type: {python_type}. Defaulting to string.")
            return cls.STRING


class PropertyValidator(BaseModel):
    """Represents a validator for a field in a DocSet schema."""

    pass


class SourceSpec(BaseModel):
    pass


class Property(BaseModel):
    model_config = ConfigDict(use_attribute_docstrings=True)

    # Not clear how to declare the type for this without explicitly
    # enumerating all the values of DataType.
    type: Literal[str]  # type: ignore
    """The type of the property."""

    required: bool = False
    """Whether the property is required."""

    description: Optional[str] = None
    """A brief description of the property."""

    default: Optional[Any] = None
    """The default value for the property."""

    extraction_instructions: Optional[str] = None
    """Additional instructions (prompts) to use when extracting the property."""

    examples: Optional[list[Any]] = None
    """Example values for this property."""

    source: Optional[SourceSpec] = None
    """Where to look for the field in the document.

    Defaults to the entire document.
    """

    validators: list[PropertyValidator] = []
    """Validators to apply to this property."""


class BoolProperty(Property):
    type: Literal[DataType.BOOL] = DataType.BOOL


class IntProperty(Property):
    type: Literal[DataType.INT] = DataType.INT


class FloatProperty(Property):
    type: Literal[DataType.FLOAT] = DataType.FLOAT


class StringProperty(Property):
    type: Literal[DataType.STRING] = DataType.STRING


class DateProperty(Property):
    type: Literal[DataType.DATE] = DataType.DATE

    format: str = "YYYY-MM-DD"


class DateTimeProperty(Property):
    type: Literal[DataType.DATETIME] = DataType.DATETIME

    # TODO: Do we care about microseconds or timezones here? Hoping to use datetime.isoformat().
    format: str = "YYYY-MM-DDTHH:MM:SS"


class ArrayProperty(Property):
    type: Literal[DataType.ARRAY] = DataType.ARRAY

    item_type: "PropertyType"


class ChoiceProperty(Property):
    type: Literal[DataType.CHOICE] = DataType.CHOICE

    choices: list[Any]


class CustomProperty(Property):
    type: Literal[DataType.CUSTOM] = DataType.CUSTOM

    custom_type: str


class NamedProperty(BaseModel):
    model_config = ConfigDict(use_attribute_docstrings=True)

    name: str
    """The name of the property."""

    type: "PropertyType"
    """The type of the property."""


class ObjectProperty(Property):
    type: Literal[DataType.OBJECT] = DataType.OBJECT
    properties: list[NamedProperty]


PropertyType: TypeAlias = Annotated[
    (
        BoolProperty
        | IntProperty
        | FloatProperty
        | StringProperty
        | DateProperty
        | DateTimeProperty
        | ArrayProperty
        | ChoiceProperty
        | CustomProperty
        | ObjectProperty
    ),
    Field(discriminator="type"),
]


def make_property(**kwargs) -> PropertyType:
    return TypeAdapter(PropertyType).validate_python(kwargs)


def make_named_property(name: str, **kwargs) -> NamedProperty:
    """Create a NamedProperty with the given name and property type."""
    return NamedProperty(name=name, type=make_property(**kwargs))


def _convert_to_named_property(schema_prop: SchemaField) -> NamedProperty:
    """Convert a SchemaProperty to a NamedProperty."""

    prop_type_dict = {
        "type": schema_prop.field_type,
        "default": schema_prop.default,
        "description": schema_prop.description,
        "examples": schema_prop.examples,
    }

    if (declared_type := prop_type_dict["type"]) not in DataType.values():
        prop_type_dict["custom_type"] = declared_type
        prop_type_dict["type"] = DataType.CUSTOM

    return NamedProperty(
        name=schema_prop.name,
        type=make_property(**prop_type_dict),
    )


def _validate_new_schema(v: Any, handler: ValidatorFunctionWrapHandler) -> NamedProperty:
    try:
        return handler(v)
    except ValidationError:
        # Attempt to validate as a SchemaProperty and convert to NamedProperty
        schema_prop = SchemaField.model_validate(v)
        return _convert_to_named_property(schema_prop)


# @experimental
class SchemaV2(BaseModel):
    """Represents the schema of a DocSet."""

    properties: list[Annotated[NamedProperty, WrapValidator(_validate_new_schema)]] = Field(
        description="A list of properties belonging to this schema.",
        validation_alias=AliasChoices("properties", "fields"),
    )

    @property
    def fields(self) -> list[NamedProperty]:
        """Alias for properties."""
        return self.properties

    def flatten(self) -> "SchemaV2":
        """Flatten the schema by removing nested properties."""

        def flatten_object(prefix: str, props: list[NamedProperty], out_props: list[NamedProperty]) -> None:
            """Flatten an ObjectProperty into its properties."""
            for p in props:
                if p.type.type == DataType.ARRAY:
                    continue
                elif p.type.type == DataType.OBJECT:
                    # Flatten nested object properties
                    new_prefix = f"{prefix}.{p.name}" if prefix else p.name
                    flatten_object(new_prefix, p.type.properties, out_props)
                else:
                    new_p = p.model_copy(deep=True)
                    if len(prefix) > 0:
                        new_p.name = f"{prefix}.{p.name}"
                    out_props.append(new_p)

        flattened_properties: list[NamedProperty] = []
        flatten_object("", self.properties, flattened_properties)
        return SchemaV2(properties=flattened_properties)

    def render_flattened(self) -> str:
        flattened = self.flatten()
        props = [
            {"name": p.name, **p.type.model_dump(exclude_unset=True, exclude_none=True)} for p in flattened.properties
        ]
        return json.dumps({"properties": props}, indent=2)



import json
import tempfile
from pathlib import Path

from aryn_sdk.partition import draw_with_boxes, tables_to_pandas, partition_file
from mcp.server.fastmcp import FastMCP
from .aryn_docset_manager import ArynDocSetManager
from .aryn_document_manager import ArynDocumentManager
from .utils.utils import save_file, get_output_dir, create_zip_from_dataframes

from .models import (
    PartitionModel,
    DrawBoxesModel,
    CreateArynDocSetModel,
    GetArynDocSetModel,
    ListArynDocSetsModel,
    DeleteArynDocSetModel,
    AddArynDocumentModel,
    ListArynDocumentsModel,
    GetArynDocumentComponentsModel,
    GetArynDocumentExtractedPropertiesModel,
    DeleteArynDocumentModel,
    ExtractArynDocumentPropertiesModel,
    DeleteArynDocSetPropertiesModel,
    SearchArynDocSetModel,
    QueryArynDocSetModel,
)

mcp = FastMCP(
    name="ArynMCPServer",
)

ADSM = ArynDocSetManager()
ADM = ArynDocumentManager()


@mcp.tool()
def partition_pdf(args: PartitionModel) -> str:
    """Converts a document in PDF format to either JSON or Markdown using Aryn's partitioning service

    Args:
        args: The input arguments defined in the PartitionModel schema. These include:
        filename
        file
        threshold
        text_mode
        table_mode
        remove_line_breaks
        include_additional_text
        extract_images
        extract_image_format
        summarize_images
        selected_pages
        strategy
        max_tokens
        tokenizer
        merge_across_pages
        output_format
        promote_title
        title_candidate_elements
        orientation_correction
        include_pagenum
        include_headers
        include_footers
        add_to_docset_id

    Returns:
        A string describing where the result is stored and the name of the file
    """
    try:
        partition_result = partition_file(
            args.file,
            threshold=args.threshold,
            text_mode=args.text_mode,
            table_mode=args.table_mode,
            text_extraction_options={"remove_line_breaks": args.remove_line_breaks},
            table_extraction_options={
                "include_additional_text": args.include_additional_text,
            },
            extract_images=args.extract_images,
            extract_image_format=args.extract_image_format,
            summarize_images=args.summarize_images,
            selected_pages=args.selected_pages,
            chunking_options={
                "strategy": args.strategy,
                "max_tokens": args.max_tokens,
                "tokenizer": args.tokenizer,
                "merge_across_pages": args.merge_across_pages,
            },
            output_format=args.output_format,
            output_label_options={
                "promote_title": args.promote_title,
                "title_candidate_elements": args.title_candidate_elements,
                "orientation_correction": args.orientation_correction,
            },
            markdown_options={
                "include_pagenum": args.include_pagenum,
                "include_headers": args.include_headers,
                "include_footers": args.include_footers,
            },
            add_to_docset_id=args.add_to_docset_id,
        )

        save_file(partition_result, args.filename, args.output_format)

        return f"File saved in {get_output_dir()} as {args.filename}.{args.output_format}"
    except Exception as e:
        return str(e)


@mcp.tool()
def get_boxes_drawn_on_pdf(args: DrawBoxesModel) -> dict:
    """Saves a list of images from the partitioned pdf, one for each page, with bounding boxes detected by the partitioner drawn on.

    Args:
        args: The input arguments defined in the DrawBoxesModel schema. These include:
        docset_id
        doc_id
        path_to_partitioned_json
        path_to_original_pdf
        pages_to_draw_boxes_on
    Returns:
        result: A dictionary with the saved image paths and the number of images saved
    """

    try:
        if args.path_to_partitioned_json and args.path_to_original_pdf:
            with open(args.path_to_partitioned_json, "r") as f:
                partition_result = json.load(f)
            original_pdf_path = args.path_to_original_pdf
        else:
            assert args.docset_id and args.doc_id, "docset_id and doc_id are required"
            partition_result = ADM.get_document(
                docset_id=args.docset_id,
                doc_id=args.doc_id,
                include_elements=True,
                include_binary=False,
            )
            partition_result = {"elements": partition_result["original_elements"]}
            original_pdf_path = ADM.get_document_binary(
                docset_id=args.docset_id,
                doc_id=args.doc_id,
                file_path=Path(tempfile.gettempdir()) / f"{args.doc_id}.pdf",
            )

        pages = draw_with_boxes(original_pdf_path, partition_result)

        saved_images = []
        for page_range in args.pages_to_draw_boxes_on:
            if page_range.start < 1 or page_range.end > len(pages):
                raise ValueError(f"Page range {page_range} is out of bounds for the document")
            for page_index in range(page_range.start - 1, page_range.end):
                saved_path = save_file(pages[page_index], f"{args.doc_id}_page_image_{page_index+1}", "png")
                saved_images.append(saved_path)

        return {
            "saved_image_paths": saved_images,
            "saved_image_count": len(saved_images),
        }
    except Exception as e:
        return {"error": str(e)}


# =============================================================================
# Group: DocSet Managment Functions
# =============================================================================
@mcp.tool()
def create_aryn_docset(args: CreateArynDocSetModel) -> dict:
    """Creates a new Aryn DocSet to store documents

    Args:
        args: The input arguments defined in the CreateArynDocsetModel schema. These include:
        name
        schema
    Returns:
        result: A DocSetMetadata dictionary
    """

    try:
        docset_info = ADSM.create_docset(name=args.name, schema=args.document_schema)
        return docset_info
    except Exception as e:
        return {"error": str(e)}


@mcp.tool()
def get_aryn_docset_metadata(args: GetArynDocSetModel) -> dict:
    """Gets an Aryn DocSet to store documents

    Args:
        args: The input arguments defined in the GetArynDocsetModel schema. These include:
        docset_id
    Returns:
        result: A DocSetMetadata dictionary
    """

    try:
        docset_info = ADSM.get_docset(docset_id=args.docset_id, exclude_schema=True)
        assert docset_info, "Docset not found"

        return docset_info
    except Exception as e:
        return {"error": str(e)}


@mcp.tool()
def get_aryn_docset_schema(args: GetArynDocSetModel) -> str:
    """Gets the properties of an Aryn DocSet

    Args:
        args: The input arguments defined in the GetArynDocSetModel schema. These include:
        docset_id
    Returns:
        schema: A dictionary of extracted properties of the docset
    """

    try:
        docset_info = ADSM.get_docset(docset_id=args.docset_id, exclude_schema=False)
        assert docset_info, "Docset not found"

        save_file(docset_info["schema"], f"{args.docset_id}_schema", "json")

        return f"File saved in {get_output_dir()} as {args.docset_id}_schema.json"
    except Exception as e:
        return str(e)


@mcp.tool()
def list_aryn_docsets(args: ListArynDocSetsModel) -> list[dict] | dict:
    """Lists all DocSets in the account

    Args:
        args: The input arguments defined in the ListArynDocsetsModel schema. These include:
        page_size
        name_eq
    Returns:
        result: A list of DocSetMetadata dictionaries
    """

    try:
        docsets_info = ADSM.list_docsets(page_size=args.page_size, name_eq=args.name_eq, page_token=args.page_token)
        return docsets_info
    except Exception as e:
        return {"error": str(e)}


@mcp.tool()
def delete_aryn_docset(args: DeleteArynDocSetModel) -> dict:
    """Deletes an Aryn DocSet

    Args:
        args: The input arguments defined in the DeleteArynDocsetModel schema. These include:
        docset_id
    Returns:
        result: A DocSetMetadata dictionary of the deleted docset
    """

    try:
        docset_info = ADSM.delete_docset(docset_id=args.docset_id)
        return docset_info
    except Exception as e:
        return {"error": str(e)}


# =============================================================================
# Group: Document Managment Functions
# =============================================================================
@mcp.tool()
def add_aryn_document(args: AddArynDocumentModel, options: PartitionModel) -> dict:
    """Adds a document to an Aryn DocSet

    Args:
        args: The input arguments defined in the AddArynDocumentModel schema. These include:
        file
        docset_id
        options: The input arguments defined in the PartitionModel schema. These include:
        threshold
        text_mode
        table_mode
        remove_line_breaks
        include_additional_text
        extract_images
        extract_image_format
        summarize_images
        selected_pages
        strategy
        max_tokens
        tokenizer
        merge_across_pages
        promote_title
        title_candidate_elements
        orientation_correction
        include_pagenum
        include_headers
        include_footers
    Returns:
        result: A DocumentMetadata dictionary of the added document
    """

    try:
        document_info = ADM.add_document(file=args.file, docset_id=args.docset_id, options=options)
        return document_info
    except Exception as e:
        return {"error": str(e)}


@mcp.tool()
def list_aryn_documents(args: ListArynDocumentsModel) -> dict:
    """Lists all documents in an Aryn DocSet

    Args:
        args: The input arguments defined in the ListArynDocumentsModel schema. These include:
        docset_id
        page_size
        page_token
    Returns:
        result: A list of DocumentMetadata dictionaries
    """

    try:
        documents_info = ADM.list_documents(
            docset_id=args.docset_id,
            page_size=args.page_size,
            page_token=args.page_token,
        )
        return documents_info
    except Exception as e:
        return {"error": str(e)}


@mcp.tool()
def get_aryn_document_elements(args: GetArynDocumentComponentsModel) -> str:
    """Gets a document's elements from an Aryn DocSet document

    Args:
        args: The input arguments defined in the GetArynDocumentComponentsModel schema. These include:
        return_original_elements
        docset_id
        doc_id
    Returns:
        result: a string describing where the contents of the document are saved
    """

    try:
        document_dict = ADM.get_document(
            docset_id=args.docset_id,
            doc_id=args.doc_id,
            include_elements=True,
            include_binary=False,
        )

        if args.return_original_elements:
            document_elements = document_dict["original_elements"]
        else:
            document_elements = document_dict["elements"]

        save_file(document_elements, args.doc_id, "json")

        return f"File saved in {get_output_dir()} as {args.doc_id}.json"
    except Exception as e:
        return str(e)


@mcp.tool()
def get_aryn_document_extracted_properties(
    args: GetArynDocumentExtractedPropertiesModel,
) -> str:
    """Gets the extracted properties of a document from an Aryn DocSet document

    Args:
        args: The input arguments defined in the GetArynDocumentExtractedPropertiesModel schema. These include:
        docset_id
        doc_id
        output_format
    Returns:
        result: a string describing where the extracted properties are saved
    """
    try:
        document_dict = ADM.get_document(
            docset_id=args.docset_id,
            doc_id=args.doc_id,
            include_elements=False,
            include_binary=False,
        )
        document_properties = document_dict["properties"]

        save_file(document_properties, args.doc_id, args.output_format)

        return f"File saved in {get_output_dir()} as {args.doc_id}.{args.output_format}"
    except Exception as e:
        return str(e)


@mcp.tool()
def get_aryn_document_tables(args: GetArynDocumentComponentsModel) -> str:
    """Gets the tables of a document from an Aryn DocSet document

    Args:
        args: The input arguments defined in the GetArynDocumentComponentsModel schema. These include:
        docset_id
        doc_id
    Returns:
        result: a string describing where the tables are saved
    """
    try:
        document_dict = ADM.get_document(
            docset_id=args.docset_id,
            doc_id=args.doc_id,
            include_elements=True,
            include_binary=False,
        )
        elements = document_dict["original_elements"]

        tables = tables_to_pandas({"elements": elements})
        tables = [table for _, table in tables if table is not None]

        zip_data = create_zip_from_dataframes(tables)

        save_file(zip_data, args.doc_id, "zip")

        return f"File saved in {get_output_dir()} as {args.doc_id}.zip"
    except Exception as e:
        return str(e)


@mcp.tool()
def get_aryn_document_original_file(args: GetArynDocumentComponentsModel) -> str:
    """Gets the raw data of a document from an Aryn DocSet document

    Args:
        args: The input arguments defined in the GetArynDocumentComponentsModel schema. These include:
        docset_id
        doc_id
    Returns:
        result: a string describing where the original file is saved
    """

    try:
        ADM.get_document_binary(
            docset_id=args.docset_id,
            doc_id=args.doc_id,
            file_path=get_output_dir() / f"{args.doc_id}.pdf",
        )

        return f"File saved in {get_output_dir()} as {args.doc_id}.pdf"
    except Exception as e:
        return str(e)


@mcp.tool()
def delete_aryn_document(args: DeleteArynDocumentModel) -> dict:
    """Deletes document from an Aryn DocSet

    Args:
        args: The input arguments defined in the DeleteArynDocumentModel schema. These include:
        docset_id
        doc_id
    Returns:
        result: A DocumentMetadata dictionary of the deleted document or error message
    """

    try:
        document_info = ADM.delete_document(docset_id=args.docset_id, doc_id=args.doc_id)
        return document_info
    except Exception as e:
        return {"error": str(e)}


@mcp.tool()
def extract_aryn_docset_properties(args: ExtractArynDocumentPropertiesModel) -> dict:
    """Extracts properties from all documents in an Aryn DocSet

    Args:
        args: The input arguments defined in the ExtractArynDocumentPropertiesModel schema. These include:
        docset_id
        schema
    Returns:
        result: A job status of the job
    """

    try:
        extraction_status = ADSM.extract_properties(
            docset_id=args.docset_id, properties_to_extract=args.document_schema
        )
        return extraction_status
    except Exception as e:
        return {"error": str(e)}


@mcp.tool()
def delete_aryn_docset_properties(args: DeleteArynDocSetPropertiesModel) -> dict:
    """Deletes properties from all documents in an Aryn DocSet

    Args:
        args: The input arguments defined in the DeleteArynDocSetPropertiesModel schema. These include:
        docset_id
        schema
    Returns:
        result: A job status of the job
    """
    try:
        deletion_status = ADSM.delete_properties(
            docset_id=args.docset_id, properties_to_delete=args.properties_to_delete
        )
        return deletion_status
    except Exception as e:
        return {"error": str(e)}


@mcp.tool()
def search_aryn_docset(args: SearchArynDocSetModel) -> dict:
    """Search over a docset and get back documents or elements that match your search criteria

    Args:
        args: The input arguments defined in the SearchArynDocSetModel schema. These include:
        docset_id
        query_or_properties_filter
        query
        query_type
        properties_filter
        k
        return_type
        page_token
    Returns:
        result: A dict of returned attributes
    """
    try:
        search_result = ADSM.search(
            docset_id=args.docset_id,
            query_or_properties_filter=args.query_or_properties_filter,
            query=args.query,
            query_type=args.query_type,
            properties_filter=args.properties_filter,
            page_size=args.page_size,
            return_type=args.return_type,
            page_token=args.page_token,
        )

        return search_result
    except Exception as e:
        return {"error": str(e)}


@mcp.tool()
def query_aryn_docset(args: QueryArynDocSetModel) -> dict:
    """Queries an Aryn DocSet

    Args:
        args: The input arguments defined in the QueryArynDocSetModel schema. These include:
        docset_id
        query
        summarize_result
    """
    try:
        query_result = ADSM.query(
            docset_id=args.docset_id,
            query=args.query,
            summarize_result=args.summarize_result,
        )

        return query_result
    except Exception as e:
        return {"error": str(e)}


def main():
    mcp.run(transport="stdio")


if __name__ == "__main__":
    main()



from pydantic import BaseModel, Field


class ListArynDocSetsModel(BaseModel):
    """
    Input schema for list_aryn_docsets()

    Attributes:
        page_size
        name_eq
    """

    page_size: int = Field(
        100,
        description="""
            page_size (int, optional)
            the number of items per page. Default value is 100""",
    )

    name_eq: str | None = Field(
        None,
        description="""
            name_eq (str, optional)
            When provided, filters DocSets by exact name match. Value should be None if no filter is to be done.""",
    )

    page_token: str | None = Field(
        None,
        description="""
            page_token (str, optional)
            Token for pagination. If provided, this will indicate to the server where to begin the next set
            of records to return. Valid for 24 hours.""",
    )



from pydantic import BaseModel, Field
from typing import Literal
from os import PathLike


class PartitionModel(BaseModel):
    """
    Input schema for PartitionModel

    Attributes:
        filename
        file
        threshold
        text_mode
        table_mode
        remove_line_breaks
        include_additional_text
        model_selection
        extract_images
        extract_image_format
        summarize_images
        selected_pages
        strategy
        max_tokens
        tokenizer
        merge_across_pages
        output_format
        promote_title
        title_candidate_elements
        orientation_correction
        include_pagenum
        include_headers
        include_footers
    """

    filename: str = Field(
        "",
        description="""
            filename (str, required)
            Name of the file to save. No file extension should be included.""",
    )

    file: str | PathLike = Field(
        "",
        description="""
            file (str | PathLike, required)
            Path to the PDF document or an http link to a publicly accessible PDF document""",
    )

    threshold: float = Field(
        0.32,
        description="""
            threshold (float, optional)
            A float between 0.0 and 1.0, inclusive, which serves as a cutoff to determine which bounding boxes for
            the file are returned or a string auto (the default) where the service uses a processing method to find the best prediction for each
            possible bounding box. Only bounding boxes that the model predicts with a confidence score higher than the threshold specified will
            be returned. A lower value will include more objects but may have overlaps, while a higher value will reduce the number of overlaps
            but may miss legitimate objects. If you do set the threshold manually, I recommend starting with a value of 0.32.""",
    )

    text_mode: Literal["inline_fallback_to_ocr", "inline", "ocr_standard", "ocr_vision"] = Field(
        "inline_fallback_to_ocr",
        description="""
            text_mode (str, optional)
            A string that specifies the mode to use for text extraction. There are 4 possible values:
            inline_fallback_to_ocr: Tries to extract the embedded text elementwise and falls back to performing OCR otherwise. This is the default.
            inline:                 Extracts embedded text
            ocr_standard:           Uses the classical OCR pipeline
            ocr_vision:             Uses a vision model for OCR. Note that ocr_vision is only available for PAYG users only.""",
    )

    table_mode: Literal[None, "standard", "vision"] = Field(
        "standard",
        description="""
            table_mode (str | None, optional)
            A string that specifies the mode to use for table structure extraction. There are 3 possible values:
            None:      will not extract table structure
            standard:  will use the standard hybrid table structure extraction pipeline. This is the default value
            vision:    will use a vision model to extract table structure""",
    )

    remove_line_breaks: bool = Field(
        False,
        description="""
            remove_line_breaks (bool, optional)
            A boolean that specifies whether to remove line breaks from the text. Default is False.""",
    )

    include_additional_text: bool = Field(
        False,
        description="""
            include_additional_text (bool, optional)
            A value in a map with string keys specifying options for table extraction. When True, will attempt to enhance the table
            structure by merging in tokens from text extraction. This can be useful for working with tables that have missing or
            misaligned text. Default is False""",
    )

    extract_images: bool = Field(
        False,
        description="""
            extract_images (bool, optional)
            A boolean that determines whether to extract images from the document. Default value is False""",
    )

    extract_image_format: Literal["ppm", "png", "jpeg"] = Field(
        "ppm",
        description="""
            extract_image_format (str, optional)
            A string indicating what in what format extracted images should be returned. In all cases, the result will be base64
            encoded before being returned. There are 3 possible values:
            ppm: This is the default
            png
            jpeg""",
    )

    summarize_images: bool = Field(
        False,
        description="""
            summarize_images (bool, optional)
            A boolean that, when True, generates a summary of the images in the document and returns it as the text_representation.
            When False, images are not summarized. Default is False.""",
    )

    selected_pages: list[int] | None = Field(
        None,
        description="""
            selected_pages (list[int], optional)
            A list specifying individual pages (1-indexed) and page ranges from the document to partition. Single pages are
            specified as integers and ranges are specified as lists with two integer entries in ascending order. A valid example
            value for selected_pages is [1, 10, [15, 20]] which would include pages 1, 10, 15, 16, 17 …, 20. selected_pages is
            None by default, which results in all pages of the document being parsed.""",
    )

    strategy: Literal["context_rich", "maximize_within_limit"] = Field(
        "context_rich",
        description="""
            strategy (str, optional)
            A value in a dictionary of options for specifying chunking behavior. A string specifying the strategy to use to combine and split chunks.
            There are two possible values:
            context_rich:           The goal of this chunking strategy is to add context to evenly-sized chunks. This is most useful for retrieval
                                    based GenAI applications. Context_rich chunking combines adjacent section-header and title elements into a
                                    new section-header element. Merges elements into a chunk with its most recent section-header. If the chunk
                                    would contain too many tokens, then it starts a new chunk copying the section-header to the start of this new
                                    chunk and continues. Merges elements on different pages, unless merge_across_pages is set to False.
            maximize_within_limit:  The goal of the maximize_within_limit chunker is to make the chunks as large as possible. Merges elements into
                                    the last most recently merged set of elements unless doing so would make its token count exceed max_tokens. In
                                    that case, it would keep the new element separate and start merging subsequent elements into that one, following
                                    the same rule. Merges elements on different pages, unless merge_across_pages is set to False.""",
    )

    max_tokens: int = Field(
        512,
        description="""
            max_tokens (int, optional)
            A value in a dictionary of options for specifying chunking behavior. An integer specifying the cutoff for splitting chunks that are too
            large. Default value is 512.""",
    )

    tokenizer: Literal["openai_tokenizer", "character_tokenizer", "huggingface_tokenizer"] = Field(
        "openai_tokenizer",
        description="""
            tokenizer (str, optional)
            A value in a dictionary of options for specifying chunking behavior. A string specifying the tokenizer to use when determining how characters
            in a chunk are grouped. There are 3 possible values:
            openai_tokenizer: This is the default value
            character_tokenizer
            huggingface_tokenizer""",
    )

    merge_across_pages: bool = Field(
        True,
        description="""
            merge_across_pages (bool, optional)
            A value in a dictionary of options for specifying chunking behavior. A boolean that when True the selected chunker will attempt to merge chunks
            across page boundaries. Does not apply to the mixed_multi_column merger, which never merges across pages. Defaults to True.""",
    )

    output_format: Literal["markdown", "json"] = Field(
        "json",
        description="""
            output_format (str, required)
            Can be either two values:
            "markdown"
            "json""",
    )

    promote_title: bool = Field(
        False,
        description="""
            promote_title (bool, optional)
            A value in a dictionary of options to specify which heuristic to apply to enforce certain label outputs. A boolean that specifies whether to
            promote an element to title if there’s no title in the output. Default value is False""",
    )

    title_candidate_elements: list[str] = Field(
        [],
        description="""
            title_candidate_elements (list[str], optional)
            A value in a dictionary of options to specify which heuristic to apply to enforce certain label outputs. A list of strings that are candidate
            elements to be promoted to title. Default value is []""",
    )

    orientation_correction: bool = Field(
        False,
        description="""
            orientation_correction (bool, optional)
            A value in a dictionary of options to specify which heuristic to apply to enforce certain label outputs. A boolean value that specifies whether
            to correct the orientation of rotated pages during the preprocessing step. Default value is False""",
    )

    include_pagenum: bool = Field(
        False,
        description="""
            include_pagenum (bool, optional)
            A value in a dictionary of options to specify what to include in the markdown output. A boolean that specifies whether to include page numbers
            in the markdown output. Default is False.""",
    )

    include_headers: bool = Field(
        False,
        description="""
            include_headers (bool, optional)
            A value in a dictionary of options to specify what to include in the markdown output.  A boolean that specifies whether to include headers
            in the markdown output. Default is False.""",
    )

    include_footers: bool = Field(
        False,
        description="""
            include_footers (bool, optional)
            A value in a dictionary of options to specify what to include in the markdown output. A boolean that specifies whether to include footers
            in the markdown output. Default is False.""",
    )

    add_to_docset_id: str | None = Field(
        None,
        description="""
            add_to_docset_id (str | None, optional)
            The id of the Aryn DocSet the partitioned file will get added to. Default value is None. """,
    )



from pydantic import BaseModel, Field, model_validator
from typing import Literal
from .properties_filter_model import PropertiesFilterModel


class SearchArynDocSetModel(BaseModel):
    """
    Input schema for search_aryn_docset()

    Attributes:
        docset_id
        query
        query_type
        properties_filter
        k
        return_type
        page_token
    """

    docset_id: str = Field(
        ...,
        description="""
            docset_id (str, required)
            The unique identifier of the DocSet you are searching over""",
    )

    query_or_properties_filter: Literal["query", "properties_filter"] = Field(
        ...,
        description="""
            query_or_properties_filter (str, required)
            An enum that can be either 2 values:
            query:      When query is specified, the search call will use the query parameter to search over the docset.
            properties_filter:  When properties_filter is specified, the search call will use the properties to search over the docset.""",
    )

    query: str | None = Field(
        None,
        description="""
            query (str | None, optional)
            A string that specifies what term you are searching for within the contents of your documents.
            The query_type parameter specified will control exactly how this query parameter is used
            to search over your documents.""",
    )

    query_type: None | Literal["keyword", "vector", "lexical", "hybrid"] = Field(
        "lexical",
        description="""
            query_type (str, required)
            An enum that can be either 4 values:
            keyword:  When keyword is specified, the search call will perform a substring match and return
                      results that contain strings that contain the query term specified.
            vector:   When vector is specified, the search call will internally embed the query with the
                      embedding function associated with the docset you are querying on and perform a k-nearest
                      neighbor search to retrieve the results.
            lexical:  When lexical is specified, the search call will perform an exact string match and return
                      results where the query string shows up as a standalone word.
            hybrid:   A mix of vector and lexical""",
    )

    properties_filter: list[PropertiesFilterModel] | None = Field(
        None,
        description="""
            properties_filter (str | None, optional)
            A list of schemas that help create an expression that specifies the condition to use when extract
            extracting documents or elements from the docset. Default value is None""",
    )

    page_size: int = Field(
        ...,
        description="""
            page_size (int, required)
            The number of records to return back.""",
    )

    return_type: Literal["doc", "element"] = Field(
        ...,
        description="""
            return_type (str, required)
            An enum that an be either 2 values:
            doc:      When doc is specified, documents that match the search criteria are retuned.
            element:  When element is specified, specific sections of the document (i.e. elements) are returned.""",
    )

    page_token: str | None = Field(
        None,
        description="""
            page_token (str | None, optional)
            A string used for pagination purposes. If provided, this will indicate to the server where to begin the
            next set of records to return. Valid for 24 hours.""",
    )

    @model_validator(mode="after")
    def validate_search_criteria(self) -> "SearchArynDocSetModel":
        if self.query_or_properties_filter == "query" and not (self.query or self.query_type):
            raise ValueError("If query_or_properties_filter is 'query', query and query_type must be provided")
        if self.query_or_properties_filter == "properties_filter" and (not self.properties_filter):
            raise ValueError("If query_or_properties_filter is 'properties_filter', properties_filter must be provided")
        return self



from pydantic import BaseModel, Field, field_validator, model_validator


class PageRange(BaseModel):
    start: int = Field(
        ...,
        description="""
            start (int, required)
            The start page number. The page numbers are 1-indexed.""",
    )
    end: int = Field(
        ...,
        description="""
            end (int, required)
            The end page number. The page numbers are 1-indexed.""",
    )

    @model_validator(mode="after")
    def check_start_is_less_than_end(self) -> "PageRange":
        if self.start >= self.end:
            raise ValueError("Start page number must be less than the end page number")
        return self


class DrawBoxesModel(BaseModel):
    """
    Input schema for get_boxes_drawn_on_pdf()

    Attributes:
        docset_id
        doc_id
        path_to_partitioned_json
        path_to_original_pdf
        pages_to_draw_boxes_on
    """

    docset_id: str | None = Field(
        None,
        description="""
            docset_id (str, optional)
            The unique identifier of the DocSet containing the document""",
    )

    doc_id: str | None = Field(
        None,
        description="""
            doc_id (str, optional)
            The unique identifier of the document to retrieve""",
    )

    path_to_partitioned_json: str | None = Field(
        None,
        description="""
            path_to_partitioned_json (str, optional)
            A path to a partitioned pdf in json form. The partitioned result MUST be a json format.""",
    )

    @field_validator("path_to_partitioned_json")
    @classmethod
    def check_is_json(cls, v: str) -> str:
        if not v.lower().endswith(".json"):
            raise ValueError("File type must be json")
        return v

    path_to_original_pdf: str | None = Field(
        None,
        description="""
            path_to_original_pdf (str, optional)
            A path to the original pdf that was partitioned.""",
    )

    @field_validator("path_to_original_pdf")
    @classmethod
    def check_is_pdf(cls, v: str) -> str:
        if not v.lower().endswith(".pdf"):
            raise ValueError("File type must be pdf")
        return v

    pages_to_draw_boxes_on: list[PageRange] = Field(
        ...,
        description="""
            pages_to_draw_boxes_on (list[PageRange], required)
            A list of page ranges to draw boxes on. Ranges are defined like python slices where the start is inclusive and the end is exclusive.
            The page numbers are 1-indexed.""",
    )

    @model_validator(mode="after")
    def validate_mutually_exclusive_fields(self) -> "DrawBoxesModel":
        """
        Validates that either:
        1. docset_id and doc_id are provided
        2. path_to_partitioned_json and path_to_original_pdf are provided
        """
        has_docset_fields = bool(self.docset_id and self.doc_id)
        has_path_fields = bool(self.path_to_partitioned_json and self.path_to_original_pdf)

        if not has_docset_fields and not has_path_fields:
            raise ValueError(
                "Either (docset_id, doc_id) OR (path_to_partitioned_json, path_to_original_pdf) must be provided. "
                "Both sets cannot be empty."
            )

        if has_docset_fields and has_path_fields:
            raise ValueError(
                "Cannot provide both (docset_id, doc_id) AND (path_to_partitioned_json, path_to_original_pdf). "
                "Please provide only one set of fields."
            )

        return self



from pydantic import BaseModel, Field


class DeleteArynDocSetModel(BaseModel):
    """
    Input schema for delete_aryn_docset()

    Attributes:
        docset_id
    """

    docset_id: str = Field(
        ...,
        description="""
            docset_id (str, required)
            The unique identifier of the DocSet to delete""",
    )



from pydantic import BaseModel, Field
from .document_schema import Schema


class CreateArynDocSetModel(BaseModel):
    """
    Input schema for create_aryn_docset()

    Attributes:
        name
        schema
    """

    name: str = Field(
        ...,
        description="""
            name (str, required)
            The name of the Aryn DocSet""",
    )

    document_schema: Schema | None = Field(
        None,
        description="""
            schema (Schema, optional)
            A schema that defines properties to extract out of a document being parsed. For example,
            In a DocSet of quarterly reports, a property to extract would be the date, or the quarter.""",
    )



from pydantic import BaseModel, Field


class DeleteArynDocumentModel(BaseModel):
    """
    Input schema for delete_aryn_document()

    Attributes:
        docset_id
        doc_id
    """

    docset_id: str = Field(
        ...,
        description="""
            docset_id (str, required)
            The unique identifier of the DocSet containing the document""",
    )

    doc_id: str = Field(
        ...,
        description="""
            doc_id (str, required)
            The unique identifier of the document to delete""",
    )
